@@ -47,15 +47,17 @@ class Peripheral:
         warnings.warn("This API has been deprecated.", DeprecationWarning)
 
     def __repr__(self) -> str:
-        ret = f"name: {self.name}\n" + \
-              f"brand: {self.brand}\n" + \
-              f"model: {self.model}\n" + \
-              f"type: {self.type}\n" + \
-              f"available: {self.available}\n" + \
-              f"status_code: {self.status_code}\n" + \
-              f"support_code: {self.support_code}\n" + \
-              f"gateway_id: {self.gateway_id}\n" + \
-              f"gateway_mac_address: {self.gateway_mac_address}"
+        ret = (
+            f"name: {self.name}\n"
+            f"brand: {self.brand}\n"
+            f"model: {self.model}\n"
+            f"type: {self.type}\n"
+            f"available: {self.available}\n"
+            f"status_code: {self.status_code}\n"
+            f"support_code: {self.support_code}\n"
+            f"gateway_id: {self.gateway_id}\n"
+            f"gateway_mac_address: {self.gateway_mac_address}"
+        )
         return ret
 
     @classmethod
@@ -630,14 +632,16 @@ class AWSThing:
         self._support_code = None
 
     def __repr__(self) -> str:
-        ret = f"name: {self.name}\n" + \
-              f"brand: {self.brand}\n" + \
-              f"model: {self.model}\n" + \
-              f"type: {self.type}\n" + \
-              f"available: {self.available}\n" + \
-              f"status_code: {self.status_code}\n" + \
-              f"support_code: {self.support_code}\n" + \
-              f"gateway_mac_address: {self.gateway_mac_address}"
+        ret = (
+            f"name: {self.name}\n"
+            f"brand: {self.brand}\n"
+            f"model: {self.model}\n"
+            f"type: {self.type}\n"
+            f"available: {self.available}\n"
+            f"status_code: {self.status_code}\n"
+            f"support_code: {self.support_code}\n"
+            f"gateway_mac_address: {self.gateway_mac_address}"
+        )
         return ret
 
     @classmethod
@@ -646,19 +650,19 @@ class AWSThing:
         things_json : dict,
         device_names : Optional[Union[List[str], str]]
     ) -> Dict[str, object]:
-        """Use device names to pick peripheral_jsons accordingly.
+        """Use device names to pick things_json accordingly.
 
         Parameters
         ----------
-        peripherals_json : dict
-            Peripherals_json retrieved from GetPeripheralsByUser.
+        things_json : dict
+            things_json retrieved from aws_connection.GetAllDevice.
         device_names : list of str or str or None
             Device name. If None is given, all available devices will be included, by default None.
 
         Returns
         -------
         dict
-            A dict of Peripheral instances with device name key.
+            A dict of AWSThing instances with device name key.
         """
 
         things = {}
@@ -757,7 +761,7 @@ class AWSThing:
     
     @property
     def status_code(self) -> JciHitachiAWSStatus:
-        """Peripheral's status code reported by the API.
+        """Thing's status code reported by the API.
 
         Returns
         -------
@@ -773,7 +777,7 @@ class AWSThing:
     
     @property
     def support_code(self) -> JciHitachiAWSStatusSupport:
-        """Peripheral's support code reported by the API.
+        """Thing's support code reported by the API.
 
         Returns
         -------
@@ -787,18 +791,6 @@ class AWSThing:
     def support_code(self, x : JciHitachiAWSStatusSupport) -> None:
         self._support_code = x
 
-    @property
-    def supported_status(self) -> JciHitachiStatusSupport:
-        """Peripheral's supported status converted from support_code.
-
-        Returns
-        -------
-        JciHitachiStatusSupport
-            Supported status.
-        """
-
-        return self._supported_status
-
     @property
     def thing_name(self):
         return self._json["ThingName"]
@@ -861,7 +853,7 @@ class JciHitachiAWSAPI:
         self._aws_identity = None
         self._aws_credentials = None
         self._host_identity_id = None
-        self._error_counter = 0
+        self._token_refresh_counter = 0
         self._task_id = 0
 
     @property
@@ -871,7 +863,7 @@ class JciHitachiAWSAPI:
         Returns
         -------
         dict of AWSThing
-            A dict of AWSThing.
+            A dict of AWSThing instances.
         """
 
         return self._things
@@ -890,10 +882,21 @@ class JciHitachiAWSAPI:
         return self._task_id
 
     def _check_before_publish(self) -> None:
-        if self._mqtt.mqtt_events.mqtt_error_event.is_set() and self._error_counter < 5:
-            self._error_counter += 1
-            self._mqtt.mqtt_events.mqtt_error_event.clear()
-            self.login()
+        if self._token_refresh_counter >= 120: # assume we publish message every 30 seconds, we'll refresh the tokens every hour.
+            self._token_refresh_counter = 0
+            conn = aws_connection.JciHitachiAWSCognitoConnection(
+                email=self.email,
+                password=self.password,
+                print_response=self.print_response,
+            )
+            self._aws_tokens = conn.aws_tokens
+        else:
+            self._token_refresh_counter += 1
+
+        if self._mqtt.mqtt_events.mqtt_error_event.is_set():
+            if self._mqtt.mqtt_events.mqtt_error == "wssHandShakeError":
+                self._mqtt.mqtt_events.mqtt_error_event.clear()
+                self.login()
 
     def login(self) -> None:
         """Login API.
@@ -946,13 +949,11 @@ class JciHitachiAWSAPI:
             self._mqtt = aws_connection.JciHitachiAWSMqttConnection(self._aws_credentials, print_response=self.print_response)
             self._mqtt.configure()
 
-            topics = [f"{self._host_identity_id}_{thing.gateway_mac_address}/#" for thing in self._things.values()]
-            self._mqtt.connect(topics=topics)
+            topic = f"{self._host_identity_id}/#"
+            self._mqtt.connect(topics=topic)
 
             # status
-            self.refresh_status()
-            
-            return
+            self.refresh_status(refresh_support_code=True)
         else:
             raise RuntimeError(f"An error occurred when retrieving device info: {conn_status}")
     
@@ -962,7 +963,7 @@ class JciHitachiAWSAPI:
         
         self._mqtt.disconnect()
 
-    def change_password(self, new_password : str) -> None:
+    def change_password(self, new_password: str) -> None:
         """Change password. 
             Warning: 
             Use this function carefully, be sure you specify a strong enough password; 
@@ -994,7 +995,7 @@ class JciHitachiAWSAPI:
         if hitachi_conn_status != "OK" or aws_conn_status != "OK":
             raise RuntimeError(f"An error occurred when changing password: {hitachi_conn_status} {aws_conn_status}")
 
-    def refresh_status(self, device_name : Optional[str] = None) -> None:
+    def refresh_status(self, device_name : Optional[str] = None, refresh_support_code=False) -> None:
         """Refresh device status from the API.
 
         Parameters
@@ -1003,6 +1004,8 @@ class JciHitachiAWSAPI:
             Refreshing a device's status by its name.
             If None is given, all devices' status will be refreshed,
             by default None.
+        refresh_support_code : bool, optional
+            Whether or not to refresh support code.
         
         Raise
         -------
@@ -1017,28 +1020,26 @@ class JciHitachiAWSAPI:
                     thing.type == "unknown":
                 continue
 
-            self._mqtt.publish(f"{self._host_identity_id}_{thing.gateway_mac_address}/status/request", {"Timestamp": time.time()})
+            if refresh_support_code:
+                self._mqtt.publish(f"{self._host_identity_id}/{self._host_identity_id}_{thing.gateway_mac_address}/registration/request", {"Timestamp": time.time()})
+                if not self._mqtt.mqtt_events.device_support_event.wait(timeout=10.0):
+                    raise RuntimeError(
+                        f"An error occurred when refreshing support code."
+                    )
+                else:
+                    thing.support_code = self._mqtt.mqtt_events.device_support.get(thing.thing_name)
+                self._mqtt.mqtt_events.device_support_event.clear()
+                time.sleep(0.5)
+
+            self._mqtt.publish(f"{self._host_identity_id}/{self._host_identity_id}_{thing.gateway_mac_address}/status/request", {"Timestamp": time.time()})
             if not self._mqtt.mqtt_events.device_status_event.wait(timeout=10.0):
-                device_status = None
+                raise RuntimeError(
+                    f"An error occurred when refreshing status code."
+                )
             else:
-                device_status = self._mqtt.mqtt_events.device_status.get(thing.thing_name)
+                thing.status_code = self._mqtt.mqtt_events.device_status.get(thing.thing_name)
             self._mqtt.mqtt_events.device_status_event.clear()
-
-            time.sleep(0.5)
-            self._mqtt.publish(f"{self._host_identity_id}_{thing.gateway_mac_address}/registration/request", {"Timestamp": time.time()})
-            if not self._mqtt.mqtt_events.device_support_event.wait(timeout=10.0):
-                device_support = None
-            else:
-                device_support = self._mqtt.mqtt_events.device_support.get(thing.thing_name)
-            self._mqtt.mqtt_events.device_support_event.clear()
-
-            if device_status and device_support:
-                thing.status_code = device_status
-                thing.support_code = device_support
-                return
-            raise RuntimeError(
-                f"An error occurred when refreshing status: {device_status} {device_support}"
-            )
+            
     
     def get_status(self, device_name: Optional[str] = None, legacy=False) -> Dict[str, JciHitachiAWSStatus]:
         """Get device status after refreshing status.
@@ -1069,7 +1070,7 @@ class JciHitachiAWSAPI:
                 statuses[name] = thing.status_code
         return statuses
 
-    def set_status(self, status_name : str, status_value : int, device_name : str) -> bool:
+    def set_status(self, status_name: str, status_value: int, device_name: str) -> bool:
         """Set status to a thing.
 
         Parameters
@@ -1093,8 +1094,6 @@ class JciHitachiAWSAPI:
         """
 
         self._check_before_publish()
-        #if not self._peripherals[device_name].available:
-        #    return False
 
         thing_name = self._things[device_name].thing_name
         thing_type = self._things[device_name].type
@@ -1107,7 +1106,7 @@ class JciHitachiAWSAPI:
         #self._mqtt.mqtt_events.device_status_event.wait(timeout=5.0)
         #self._mqtt.mqtt_events.device_support_event.wait(timeout=5.0)
 
-        self._mqtt.publish(f"{self._host_identity_id}_{gateway_mac_address}/control/request", {
+        self._mqtt.publish(f"{self._host_identity_id}/{self._host_identity_id}_{gateway_mac_address}/control/request", {
             "Condition": {
                 "ThingName": thing_name,
                 "Index": 0,
