@@ -11,6 +11,7 @@ import httpx
 #import awsiot
 #from awsiot import mqtt_connection_builder
 from AWSIoTPythonSDK.MQTTLib import AWSIoTMQTTClient
+from AWSIoTPythonSDK.exception.AWSIoTExceptions import connectTimeoutException, publishTimeoutException, subscribeTimeoutException
 from .model import JciHitachiAWSStatus, JciHitachiAWSStatusSupport
 
 AWS_COGNITO_REGION = "ap-northeast-1"
@@ -19,11 +20,8 @@ AWS_COGNITO_ENDPOINT = f"cognito-identity.{AWS_COGNITO_REGION}.amazonaws.com/"
 AWS_COGNITO_CLIENT_ID = "7kfnjsb66ei1qt5s5gjv6j1lp6"
 AWS_COGNITO_USERPOOL_ID = "ap-northeast-1_aTZeaievK"
 
-
 AWS_IOT_ENDPOINT = "https://iot-api.jci-hitachi-smarthome.com"
 AWS_MQTT_ENDPOINT = "a8kcu267h96in-ats.iot.ap-northeast-1.amazonaws.com"
-AWS_POOL_ID = "ap-northeast-1:3bb17635-a6c0-431f-b484-b8de026544cf"
-AWS_GATEWAY_KEY = "606153a77f347c9b7becfeda37edb2ccde46ce863c7ac"
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -51,7 +49,6 @@ class JciHitachiMqttEvents:
     device_support: Dict[str, JciHitachiAWSStatusSupport] = field(default_factory=dict)
     device_control: Dict[str, dict] = field(default_factory=dict)
     mqtt_error: str = field(default_factory=str)
-    #device_access_time: Dict[str, int] = field(default_factory=dict)
     device_status_event: threading.Event = field(default_factory=threading.Event)
     device_support_event: threading.Event = field(default_factory=threading.Event)
     device_control_event: threading.Event = field(default_factory=threading.Event)
@@ -526,15 +523,15 @@ class JciHitachiAWSMqttConnection:
 
         splitted_topic = message.topic.split('/')
 
-        thing_name = splitted_topic[0]
+        thing_name = splitted_topic[1]
 
-        if len(splitted_topic) >= 3 and splitted_topic[1] == "status" and splitted_topic[2] == "response":
+        if len(splitted_topic) >= 4 and splitted_topic[2] == "status" and splitted_topic[3] == "response":
             self._mqtt_events.device_status[thing_name] = JciHitachiAWSStatus(payload)
             self._mqtt_events.device_status_event.set()
-        elif len(splitted_topic) >= 3 and splitted_topic[1] == "registration" and splitted_topic[2] == "response":
+        elif len(splitted_topic) >= 4 and splitted_topic[2] == "registration" and splitted_topic[3] == "response":
             self._mqtt_events.device_support[thing_name] = JciHitachiAWSStatusSupport(payload)
             self._mqtt_events.device_support_event.set()
-        elif len(splitted_topic) >= 3 and splitted_topic[1] == "control" and splitted_topic[2] == "response":
+        elif len(splitted_topic) >= 4 and splitted_topic[2] == "control" and splitted_topic[3] == "response":
             self._mqtt_events.device_control[thing_name] = payload
             self._mqtt_events.device_control_event.set()
 
@@ -551,19 +548,20 @@ class JciHitachiAWSMqttConnection:
 
         splitted_topic = topic.split('/')
 
-        thing_name = splitted_topic[0]
+        thing_name = splitted_topic[1]
 
-        if len(splitted_topic) >= 3 and splitted_topic[1] == "status" and splitted_topic[2] == "response":
+        if len(splitted_topic) >= 4 and splitted_topic[2] == "status" and splitted_topic[3] == "response":
             self._mqtt_events.device_status[thing_name] = JciHitachiAWSStatus(payload)
             self._mqtt_events.device_status_event.set()
-        elif len(splitted_topic) >= 3 and splitted_topic[1] == "registration" and splitted_topic[2] == "response":
+        elif len(splitted_topic) >= 4 and splitted_topic[2] == "registration" and splitted_topic[3] == "response":
             self._mqtt_events.device_support[thing_name] = JciHitachiAWSStatusSupport(payload)
             self._mqtt_events.device_support_event.set()
-        elif len(splitted_topic) >= 3 and splitted_topic[1] == "control" and splitted_topic[2] == "response":
+        elif len(splitted_topic) >= 4 and splitted_topic[2] == "control" and splitted_topic[3] == "response":
             self._mqtt_events.device_control[thing_name] = payload
             self._mqtt_events.device_control_event.set()
 
     def _on_message(self, topic, payload, dup, qos, retain, **kwargs):
+        return
         try:
             payload = json.loads(payload.decode())
         except Exception as e:
@@ -574,8 +572,6 @@ class JciHitachiAWSMqttConnection:
         #    print(f"{topic} {str(payload)}")
         
         splitted_topic = topic.split('/')
-
-        return
     
     def configure(self):
         """Configure MQTT.
@@ -591,9 +587,9 @@ class JciHitachiAWSMqttConnection:
             self._aws_credentials.session_token
         )
         self._mqttc.configureOfflinePublishQueueing(-1)  # Infinite offline Publish queueing
-        self._mqttc.configureDrainingFrequency(10)  # Draining: 2 Hz
+        self._mqttc.configureDrainingFrequency(10)  # Draining: 10 Hz
         self._mqttc.configureConnectDisconnectTimeout(10)  # 10 sec
-        self._mqttc.configureMQTTOperationTimeout(15)  # 5 sec
+        self._mqttc.configureMQTTOperationTimeout(15)  # 15 sec
 
     def connect(self, topics):
         """Connect to the MQTT broker and start loop.
@@ -606,6 +602,8 @@ class JciHitachiAWSMqttConnection:
         
         try:
             self._mqttc.connect()
+        except connectTimeoutException as e:
+            _LOGGER.error('Connection timed out.')
         except Exception as e:
             self._mqtt_events.mqtt_error = e.__class__.__name__
             self._mqtt_events.mqtt_error_event.set()
@@ -617,6 +615,8 @@ class JciHitachiAWSMqttConnection:
         for topic in topics:
             try:
                 self._mqttc.subscribe(topic, 1, self._on_publish)
+            except subscribeTimeoutException as e:
+                _LOGGER.error('Subscription timed out.')
             except Exception as e:
                 self._mqtt_events.mqtt_error = e.__class__.__name__
                 self._mqtt_events.mqtt_error_event.set()
@@ -633,7 +633,14 @@ class JciHitachiAWSMqttConnection:
             Payload to publish.
         """
 
-        self._mqttc.publish(topic, json.dumps(payload), 1)
+        try:
+            self._mqttc.publish(topic, json.dumps(payload), 1)
+        except publishTimeoutException as e:
+            _LOGGER.error('Publish timed out.')
+        except Exception as e:
+            self._mqtt_events.mqtt_error = e.__class__.__name__
+            self._mqtt_events.mqtt_error_event.set()
+            _LOGGER.error('Publish failed with exception: {}'.format(e))
 
     def disconnect(self):
         """Disconnect from the MQTT broker.
@@ -702,5 +709,10 @@ class JciHitachiAWSMqttConnection:
             Payload to publish.
         """
 
-        publish_future, _ = self._mqttc.publish(topic, json.dumps(payload), awscrt.mqtt.QoS.AT_LEAST_ONCE)
-        print(publish_future.result())
+        try:
+            publish_future, _ = self._mqttc.publish(topic, json.dumps(payload), awscrt.mqtt.QoS.AT_LEAST_ONCE)
+            print(publish_future.result())
+        except Exception as e:
+            self._mqtt_events.mqtt_error = e.__class__.__name__
+            self._mqtt_events.mqtt_error_event.set()
+            _LOGGER.error('Publish failed with exception: {}'.format(e))
